;=======================================================================================================
; HIDUART.ASM
;
; MOTO Development Group, Inc.
; HIDUART for the Cypress CY7C63x01 USB Microcontroller.  This code is
; based on Cypress Semiconductor's firmware frameworks code.
; A USB Human Interface Device Class Asynchronous Serial Transceiver
; Copyright (C) 2001  MOTO Development Group
;
; This program is free software; you can redistribute it and/or
; modify it.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
;
; Contacting MOTO Development Group:
;
; http://www.moto.com
; 
; MOTO Development Group
; 85 Second Street
; Suite 710
; San Francisco, CA 94105-3459
; 
; 415 281 4800 Tel
; 415 281 3809 Fax
;
; marketing@moto.com
;
;
;=======================================================================================================

CPU	63001

;DEFINE SOFTWARE_FLOW_CONTROL
; soft flow control is not working following my hack and slash changes.
; flow off and on needs to be linked to the ep1 transmit routine

include		"usb1regs.inc"            
include		"ram.inc"

;========================================
;	
;	Interrupt Vectors
;	
;========================================

org  00h  
       
	jmp  reset						; Reset vector
	jmp  Isr128						; 128us interrupt
	jmp  Isr1ms						; 1024ms interrupt
	jmp  IsrEp0						; endpoint 0 interrupt
	jmp  IsrEp1						; endpoint 1 interrupt
	jmp  reset						; reserved interrupt
	jmp  SerialReceiveByte	        ; general purpose I/O interrupt (serial rx startbit)
	jmp  IsrCext					; Wakeup_ISR or resume interrupt

;========================================
;
; reset processing
;	
;========================================
;
; Set all SFRs to their initial states
; Clear ram
;	
;========================================

reset:
	
    mov a, DATA_STACK_POINTER_START  ;Set the initial value of the data stack pointer
    swap a, dsp

	iord scr
	and a, RESET_MASK
	cmp a, POR_B                    ;Test for POR Reset
	jz por_reset

	and A, USBR_B					;Test for USB Reset
	jnz bus_reset

    mov a,00h
    iowr cext                       ; SUSPEND line goes low

	jmp wdog_reset



;========================================
;	
;	128us Interrupt
;	
;========================================
;	
;	{
;	If (the previous 128us interrupt has been missed)
;		{make appropriate corrections}
;	If (the timer has rolled over)
;		{increment timer msb}
;	Return from Interrupt
;	}
;
;========================================

Isr128:
	push a
.Reti:
	mov a, [interrupt_mask]				;
	ipret gier						;

;========================================
;	
;	Isr1ms
;	
;========================================
;	
;	{
;	
;	Check and flag missed 128us interrupts
;	If (no bus activity in last 1ms)
;		inc inactivity timer 
;	}
;
;========================================


Isr1ms:
  IF 0
	push a

	mov a, [interrupt_mask]				
	ipret gier						

  ENDIF
;========================================
;	
;	IsrEp0
;	
;========================================
;	
;	{
;	}
;
;========================================

IsrEp0:
    push a 

    mov a,0
    mov [suspend_counter], a
    
	iord EP0_RX_Status		;Read the EP0 RX Status register and check if the SETUP bit     
	and a, SETUP_B			;is set.
	jz NotSetup				;If the SETUP bit is not set then jump to NotSetup				

    call DecodeRequest      ;Parse the SETUP packet.
	
	jmp Reti	
 

NotSetup:
	iord EP0_RX_Status		
	and a, OUT_B		    ; If its not an OUT packet then jump to next test.
	jz InPacketReceived
	
	;THE PACKET IS AN OUT. CHECK TO SEE IF IT IS A NULL OUT.  IF IT IS A NULL OUT,
	;IGNORE IT AND RETURN FROM THE INTERRUPT.

	iord EP0_RX_Status
	and a,D0h				; The data count will always be 2 or greater. (The count includes			
	jz Reti				    ; the CRC for the data).  We can determine the count by checking to
							; see if bits 7,6,4 are set in the EP0_RX_Status register.

    iord gier               ; disable EP0 interrupts while we process the OUT pkt
    and a, ~EP0IE_B
    iowr gier

    iord USB_SCR                            ; Don't let OUT pkts come in
    and a, ~ENOUTS_B
    iowr USB_SCR
    
    ; at this point we've received an OUT with some data in it.
	call ControlWriteRamDataStage

	jmp Reti
	
InPacketReceived:	
	
	; We do not have a SETUP packet or an OUT packet. What else is left?  Perhaps it is an IN?
    
	iord EP0_TX_Config				; do not continue unless xfer of last paket is complete
	and a, INEN_B					; note the Input Enable bit is cleared when the host ACKs
	jnz Reti

    mov a, [current_reportid]           ; was this IN token the handshake of a TRANSMIT report?
    cmp a, RID_TRANSMIT                 ;  (are we completely done transmitting?)
    jz ep0_reportdone

    cmp a, RID_COMMAND                  ; (are we completely done with the command report?)
    jnz readINdata                      ;    host is looking for data (enumeration)    

ep0_reportdone:    

    mov a, ~BUSY_TRANSMITTING               ; inform background that we're done with tx cycle hogging
    and [fwcr], a                           ;  - background can resume looking for suspend condition

    mov a, SERIAL0RXBIT                     ; enable startbit interrupts
	iowr P0_interrupt                       ; 

    mov a, ENUMERATED_MASK                  ; re-enable startbit interrupt
    mov [interrupt_mask],a

; here would be a reasonable place to turn flow back on
IFDEF SOFTWARE_FLOW_CONTROL
    mov a,XON                               ;Load the serial register with an XON
    mov [serial_tx_reg],a                   ;and transmit it to the device
    call SerialTransmitByte
ENDIF

    FLOW_ON                             ; enable flow control (hardware)


    jmp Reti


readINdata:							
	call ControlReadDataStage			; continue sending control read data packets

Reti:
	mov a, [interrupt_mask]				; enable endpoint zero interrupts
	ipret gier						; 


;========================================
;	
;	IsrEp1
;	
;
;========================================
;
;   The EP1 Interrupt has been disabled so that it's FIFO may be used for extra RAM
;   variable storage.
;	
;	{
;	invert the data toggle bit
;	}
;
;========================================

IsrEp1:
	push a

	iord EP1_TX_Config
	xor a, DATA10_B
	iowr EP1_TX_Config

	mov a, [interrupt_mask]				
	ipret gier						

;========================================
;	
;	IsrGpio
;	
;========================================
;	
;	{
;	}
;
;========================================
; now we call directly to serialrxbyte
;IsrGpio:
;	push a
;	mov a, [interrupt_mask]				
;	ipret gier						


;========================================
;	
;	IsrCext
;
;	This interrupt is not used in this code
;	
;========================================
;	
;	{
;	}
;
;========================================

IsrCext:
	push a; 
	mov a, [interrupt_mask]
	ipret gier


;========================================
;	
;	process_reset
;	
;========================================
;	
;	{
;	}
;
;========================================

por_reset:
	mov a, ffh
	iowr P0_data					    ;set all ports to output 1
	iowr P1_data           

    mov a, 00h
    iowr P0_interrupt
	iowr P1_interrupt      
	iowr P0_pullup					    ;set pullups on and interrupt polarity low-going          
	iowr P1_pullup          

	mov a, 0fh						    ;set all Isink regs to highest sink value
	iowr P0_sink0
	iowr P0_sink1					    ;this can be replaced by an iowx loop
	iowr P0_sink2					    ;by the user, but is done "long hand"
	iowr P0_sink3					    ;here for ease of user modification.
	iowr P0_sink4	
	iowr P0_sink5	
	iowr P0_sink6	
	iowr P0_sink7
	
	iowr P1_sink0	
	iowr P1_sink1	
	iowr P1_sink2	
	iowr P1_sink3	
	iowr P1_sink4	
	iowr P1_sink5	
	iowr P1_sink6	
	iowr P1_sink7	

	iowr wdt                   			; clear wdt timer

	mov a, bottom_of_ram
	mov x, top_of_ram
ram_clr_loop:						    ; clear all ram locations
	mov [x+0], a
	dec x
	jnc ram_clr_loop

	iord scr
	and a, USBR_B
	jnz bus_reset
								
suspend_reset:
	iowr wdt

    mov a,01h
    iowr cext                           ;Set the cext pin high; indicate we're going into suspend

	mov A, (RUN_B + SUSPEND_B)
	iowr scr						    ; suspend the mcu
	nop
	jmp suspend_reset					; wait until real bus reset

wdog_reset:
	jmp suspend_reset					;suspend and wait for USB Bus Reset

bus_reset:
	mov A, RUN_B					    ; clear all reset bits
	iowr scr

    mov a,00h                           ; clear SUSPEND line 
    iowr cext 

	mov A, ENUMERATE_MASK
	mov [interrupt_mask],A
	iowr gier          

;========================================
;	
;	project-specific initialisation
;	
;========================================

    ;FIRMWARE VARIABLE INITIALIZATION
    ;------------------------------------------

    mov a, 0                        ; Init the rx offset to the start of the buffer
    mov [rx_inptr],a 	            ; circular buffer for collecting serial bytes
    mov [rx_outptr],a 	            ; and holding them until USB is ready for them
	
	mov A, 00h
	mov [serial_buf_datacount],A	;Init the number of data bytes in the rx buffer to zero

    ;CLEAR fwcr (Firmware Control Register)
    mov A,00h
    mov [fwcr],A          

    ;SET INITIAL BAUDRATE 
    mov a,BAUD_BITDELAY_DEFAULT
    mov [baud_bitdelay],a           ;Program the whole-bit delay

    ;GPIO INITIALIZATION
    ;------------------------------------------


    ;PORT 0 - PROGRAM PORT INPUT/OUTPUT/PULLUPS
    mov a,  P0PULLUPS                   ;P0.*, Enable pullups on all
    iowr    P0_pullup

    mov a,  0FDh                        ;P0.0=OUT, Config P0.1=IN, all others OUT
	iowr    P0_data

    ;PORT 0 - APPLICATION SPECIFIC PIN SETTINGS
    iord P0_data
    or a,(SERIAL0TXBIT+SERIAL0RXBIT)    ;Set TX pin to 5V 

    iowr P0_data                        

    iord P1_data
    and a, ~nRST                        ;Set reset LOW (active)
    iowr P1_data

    FLOW_OFF
    
    DELAY10MS                           ;Hold reset down for a while.

    iord P1_data
    or a, nRST                          ;Set Reset high and start-up the unit, indicate we're ready to listen
    iowr P1_data

    ;PORT 1 - PROGRAM PORT INPUT/OUTPUT/PULLUPS

    mov a, 00h                          ;Enable pullups
    iowr P1_pullup

    ;PORT 1 - APPLICATION SPECIFIC INITIAL PIN SETTINGS

holdoff:
    iowr wdt

    mov A, [interrupt_mask]             ; ep0 interrupts will set the interrupt mask to ENUMERATED
    cmp A, ENUMERATED_MASK              ;  once device is mostly enumerated and configuration is established.
    jnz holdoff

    FLOW_ON                             ; indicate clear to send (we're ready to rx)

    ; other configuration
	mov a, SERIAL0RXBIT                 ; enable GPIO interrupts on P0.1 (serial rx pin)
	iowr P0_interrupt

	mov a, 0fh						    ;set all Isink regs to highest sink value
	iowr P0_sink0


UserReset:

;========================================
;	
;	main
;	
;========================================
	

main:
	iowr wdt                    ; kick the dog

    call SuspendCounter         ; count absence of usb activity
	call CheckSuspend           ; see if USB has stopped -- we'll need to suspend

 	iord EP1_TX_Config          ; if there is an unACKed tx already flagged
	and a, INEN_B               ; then do not check for an update, as there
	jnz	main                    ; is no safe way to update until the TX has happened.

    mov a, [rx_inptr]
    cmp a, [rx_outptr]
    jz main

    mov a, 0
    mov [ep1_byte_count], a

mgetbyte:
    mov x, [rx_outptr]                  ; point to the byte received by the uart
    mov a, [X+RX_BUF_PTR_START]         ; get the next byte received by the uart

    inc X                               ; point to next posn in buffer
    swap a, X                           ; save the byte (in x), get index in a
    cmp a, SERIAL_BUF_SIZE              ; has buffer index reached end?
    jnz .updptr                         ;    no, just go ahead up keep it
    mov a, 0                            ;    yes, zero buffer index
.updptr:
    mov [rx_outptr], a

    swap a, x                           ; get the received byte in a
    mov x, [ep1_byte_count]             ; get our count/offset into the ep1 fifo
    mov [X+(EP1_FIFO+2)], a             ; place the byte in the FIFO

    inc [ep1_byte_count]                ; increment our byte count/index
    mov a, [ep1_byte_count]             ; get it in A for testing
    cmp a, 6                            ; 1 RID + 1 length + 6 data
    jz msendem                          ; we've done as many as we can do, send them

    mov a, [rx_inptr]                   ; are there any more bytes in the circ buffer?
    cmp a, [rx_outptr]
    jnz mgetbyte                        ;   yes, go get them

msendem:
    mov a, RID_RECEIVE          ; Load the FIFO and update stored data
	mov [EP1_FIFO], a
    mov a, [ep1_byte_count]     ; a count of the valid data bytes that follow
    mov [EP1_FIFO+1], a

	iord EP1_TX_Config			; Enable EP1 TX and set the byte count
	and a, DATA10_B				; preserve data toggle bit
	or a, (INEN_B + EP1EN_B + (EP1_REPORT_LENGTH + 1))
	iowr EP1_TX_Config	

    FLOW_ON                     ; we've sent some off.  get some more.

	jmp main

;========================================
SuspendCounter:
;
;
; implement a main loop suspend counter
;
    mov a, [fwcr]               ; is foreground doing a lot of uart tx work?
    and a, BUSY_TRANSMITTING    ; (if so, our sampling of timer is not very reliable,
    jnz sc_exit                 ;  and we don't need to worry about suspend anyway)

    iord timer
    rlc
    jc lookforhigh
lookforlow:
    mov a, TIMER_SYNC
    and a, [fwcr]
    jnz sc_exit

    mov a, TIMER_SYNC
    or [fwcr], a

    jmp testactivity

lookforhigh:
    mov a, TIMER_SYNC
    and a, [fwcr]
    jz sc_exit

    mov a, ~TIMER_SYNC
    and [fwcr], a

testactivity:    
	iord USB_SCR
	and a, BUSACT_B
	jz .IncCtr                              ; there hasn't been any usb activity

    iord gier                       ; collect global int enable bits
    push a                          ; save them 
    mov a, 0                    
    iowr gier                       ; disable ints to protect USB_SCR
	iord USB_SCR                    
	and a, ~BUSACT_B                ; clear activity bit
	iowr USB_SCR 
    pop a                           ; restore ints
    iowr gier					

	mov a, 00h                      ; zero counter because we've seen activity
	mov [suspend_counter], a
	jmp sc_exit

.IncCtr:
	inc [suspend_counter]           ; we've seen no activity.  inc suspend count

sc_exit:

    ret
     
;========================================
;	
;	Check Suspend
;	
;========================================
;	
;	{
;	if (time since last usb activity is > suspend time)
;		{
;		set io ports into suspend state
;		set interrupts into suspend state
;		while (no bus activity)
;			suspend
;		restore interrupts
;		restore io state
;		}
;	return
;	}
;
;========================================

CheckSuspend:

    ;CHECK THE SUSPEND TIMER TO SEE HOW LONG SINCE LAST BUS ACTIVITY
	mov a, [suspend_counter]	
	sub a, SUSPEND_TIME			
	jc .Ret					    

    ;CLEAR THE SYSPEND COUNTER TO AVOID ANY RISK WE MIGHT SUSPEND AGAIN BEFORE 1MS ISR CLEARS IT ON WAKEUP
	mov a, 00h				
	mov [suspend_counter], a	


    ;GENERIC SUSPEND OPERATIONS
    ;SAVE THE IO PORT STATUS
	iord P0_data			
	push a					
	iord P1_data				
	push a						

	iord P0_pullup			    ; this instruction appears not to work on the 3650.
	push a					    ;   the 3650 thinks P0 pullup is at 9h, manual and code say 8.

	iord P1_pullup				
	push a					

	mov a, ffh					; to ensure minimum current drain during suspend
	iowr P0_data				; all ports must be logic 1 with pullup on unless
    iowr P1_data				; externally driven.

	mov a, 00h					;	
	iowr P0_pullup				;
	iowr P1_pullup				; user to modify these values depending on circuit

    mov a,01h
    iowr cext                   ; Set the cext pin high; indicate we're going into suspend

	iord gier					; Set which interrupts may cause wake from suspend
	and a, SUSPEND_MASK			; 
	iowr gier					;

	iord scr					; *****  put the device into suspend  *****
	or a, SUSPEND_B				;
	iowr scr					;

	nop						    ; instruction that the code wakes up to

	;RESTORE PORT IO STATUS
    mov a,00h
    iowr cext                   ; Set the cext pin low; indicate we're out of suspend

    pop a						
	iowr P1_pullup				

	pop a						
    mov a, P0PULLUPS            ; doesn't matter what it was before, this is what it should be
	iowr P0_pullup				

	pop a						
	iowr P1_data				
	pop a						
	iowr P0_data				

    ;END OF GENERIC SUSPEND OPERATIONS

	mov a, [interrupt_mask]			
	iowr gier					


.Ret:
	ret


;========================================
;	
;	Not Enumerated
;	
;========================================
;	
;	This module is run if enumeration is incomplete
;
;========================================

NotEnumerated:

	jmp main


; ==============================================
; on entry rid command parameter is in A
handle_rid_command:
    cmp a, (end_baud_table-baud_table)
    jc hrc_lookup
    mov a, 0                        ; index error, just set to default
hrc_lookup:
    index baud_table
    mov [baud_bitdelay],a           ;Program the whole-bit delay
    ret

        XPAGEOFF

baud_table:
    db BAUD_BITDELAY_DEFAULT
    db BAUD_BITDELAY_2400
    db BAUD_BITDELAY_4800
    db BAUD_BITDELAY_9600
    db BAUD_BITDELAY_19200
    db BAUD_BITDELAY_38400
    db BAUD_BITDELAY_57600
end_baud_table:

        XPAGEON

    org 302h

;=-=-=-=-=-=-=-=-=-=-=-=-= Serial Port Routines =-=-=-=-=-=-=-=-=-=-=-=-=
;
;   SerialTransmitByte and SerialReceiveByte are programmable for 
;   different baud rates.
;
;   The RX and TX pins are programmed with weak pullups.
;   XPAGEOFF and XPAGEON is used to disable the compiler's XPAGE insertions in 
;   time-critical loops.
;
;========================================================================

;==================================================================
;	
;	SerialReceiveByte   - RECEIVE A BYTE ON THE SERIAL RX PIN
;
;   DESCRIPTION:    Receive a byte on the serial RX pin
;
;==================================================================


SerialReceiveByte:
XPAGEOFF
	push a

    iord EP0_RX_Status                  ; clear any previous indication of a setup token
    and a, ~(SETUP_B)                   ; we'll be looking for them while we collect bits
    iowr EP0_RX_Status

    push X

	mov a, 0                            ; +4 disable another startbit interrupt
	iowr P0_interrupt                   ; +5

    mov X,08h                           ;Init for reception of 8 data bits

    mov a,[baud_bitdelay] 
    or a,0                              ; clear carry
    rrc                                 ; divide bit delay by two
    inc a
    BIT_DELAY                           ; wait 1/2 bit time to center on start bit

SerialReceiveData:
    mov a,[baud_bitdelay] 
    BIT_DELAY                           ; wait just about a full bit time

    iord EP0_RX_Status
    and a, SETUP_B                      ; if a setup came down them we need to escape and handle it
    jnz srx_exit

         debug2toggle

    iord P0_data                        ; test the incoming bit
    and a,SERIAL0RXBIT                  ; test bit, also clear C
    mov a, [serial_rx_reg]              ; get the byte (doesn't effect the z flag)
    jnz GotOne

GotZero:
    nop
    jmp rotate

GotOne:
    cpl                                 ; cpl instruction always sets the carry flag
    cpl                                 ; do it twice to return the value to previous

rotate:
    rrc                                 ; rotate the latest bit (in c) into the MSB
    mov [serial_rx_reg], a

    dec X                               ; decrement our bit counter
    jnz SerialReceiveData               ; if not done, get another

XPAGEON

    push a
    mov a, FLOW_CTL_OFF                 ; if flow control is off, don't save
    and a, [fwcr]                       ; the byte we just rx'd
    pop a                               ; (doesn't affect Z)
    jnz waitbit                     

    mov X, [rx_inptr]
    mov [X+RX_BUF_PTR_START], a

    inc X                               ; point to next posn in buffer
    swap a, X
    cmp a, SERIAL_BUF_SIZE
    jnz updptr
    mov a, 0
updptr:
    mov [rx_inptr], a

    inc a
    cmp a, SERIAL_BUF_SIZE
    jnz calcptr
    mov a, 0
calcptr:                                ; we've got the next position
    cmp a, [rx_outptr]                  ; if next position is our outptr, then 
    jnz waitbit                         ; we need to tell device to stop transmitting 'til we catch up

    FLOW_OFF                            ; code in main routine will catch up and turn flow back on

waitbit:
    iord P0_data                        ; hang until stop bit
    and a,SERIAL0RXBIT
    jz waitbit

    mov a, SERIAL0RXBIT                 ; +4 enable another startbit interrupts
	iowr P0_interrupt                   ; +5

srx_exit:                               ; exit point w/o re-enabling interrupts
    pop X
    mov a, [interrupt_mask]
	ipret gier



;======================================================================
;	
; SerialTransmitByte
;
;   DESCRIPTION:    Transmits 8N1.  Transmit a single byte on the TX pin
;                   LSB first.
;    
;   USAGE:          call SerialTransmitByte
;   USES:           [serial_tx_reg]           
;          
;======================================================================

SerialTransmitByte:
    push X
    mov X,08h                           ; 8 bits to tx

    iord P0_data                        ;START BIT
    and a,~SERIAL0TXBIT                 ;Set TX pin to 0V
    or a,SERIAL0RXBIT                   ;Reprogram RX pin as an input.
    iowr P0_data                        ;Write new value to port

XPAGEOFF
    mov a,[baud_bitdelay] 
    inc a
    BIT_DELAY                           ; wait just about a full bit time

    nop                                 ; a little hand calibration

    
SerialTransmitBit:
    mov a, [serial_tx_reg]              ;5    get byte to tx
    rrc a                               ;4  5 rotate bottom bit into carry
    mov [serial_tx_reg],a               ;5  9  save the newly rotated byte
    jnc SerialTransmit0                 ;5(4) 14 if carry clear, send a 0

SerialTransmit1:
    iord P0_data                        ;5 18 Read the current state of Port
    or a,(SERIAL0TXBIT+SERIAL0RXBIT)    ;4 23 Set TX pin to 5V (ensure rx line stays high)
    iowr P0_data                        ;5 28 Write new value to port
    nop
    jmp txbit                           ;5 33

SerialTransmit0:
    iord P0_data                        ;4 19 Read the current state of Port
    and a,~SERIAL0TXBIT                 ;4 23 Set TX pin to 0V
    or a,SERIAL0RXBIT                   ;4 27 ensure RX pin stays high
    iowr P0_data                        ;5 32 Write new value to port

    nop
    nop

txbit:
    mov a,[baud_bitdelay]               ; 38/ 37
    inc a
    BIT_DELAY                           ; wait just about a full bit time

    dec X
    jnz SerialTransmitBit
XPAGEON

    iord P0_data                        ;Read the current state of Port
    or a,SERIAL0TXBIT                   ;Set TX pin to 5V 
    or a,SERIAL0RXBIT                   ;Reprogram RX pin as an input.
    iowr P0_data                        ;Write new value to port

    mov a,[baud_bitdelay]               ; stop bits
    BIT_DELAY                           ; wait just about a full bit time
    mov a,[baud_bitdelay] 
    BIT_DELAY                           ; wait just about a full bit time
    mov a,[baud_bitdelay] 
    BIT_DELAY                           ; wait just about a full bit time



    pop X
    ret                         ;Return to the calling subroutine


;================================



;======================== End Serial Port Routines=======================


include		"usbcode.asm"           ;USB and HID functionality
