;=======================================================================================================
; USBCODE.ASM
;
; MOTO Development Group, Inc.
; HIDUART for the Cypress CY7C63x01 USB Microcontroller.  This code is
; based on Cypress Semiconductor's firmware frameworks code.
; A USB Human Interface Device Class Asynchronous Serial Transceiver
; Copyright (C) 2001  MOTO Development Group
;
; This program is free software; you can redistribute it and/or
; modify it.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
;
; Contacting MOTO Development Group:
;
; http://www.moto.com
; 
; MOTO Development Group
; 85 Second Street
; Suite 710
; San Francisco, CA 94105-3459
; 
; 415 281 4800 Tel
; 415 281 3809 Fax
;
; marketing@moto.com
;
;=======================================================================================================
;       MODULE Decode Request
;
;=======================================================================================================
;   Change History
;   -------------------------
;   Revision:       0.1
;   Created DGW November 1999
;   Code structure original. Code incorpoatates fragments from many sources plus new code.
;   Issue Date:     DRAFT
;
;=======================================================================================================
; Note that this module is designed to work with a ROM table of 255 bytes or less.
; ALWAYS Check the length of the ROM table. If it is too long then it is recommended to either reduce the 
; length of the string descritors and/or split the table into two tables.


; VERY IMPORTANT Note from MOTO:
;
; This next value controls how often the host PC sends down "INTERRUPT IN" tokens.
; The firmware can only send data up to the host in response to an INTERRUPT IN token,
; so this is a direct control of the UART receive capability of the device.  If the 
; polling is too infrequent your UART receive buffer will overflow because it can't
; send data up to the host quickly enough.  If it is too frequent you may cause problems
; with other USB devices.
;
; Now, most HID devices put 10 here, to indicate "poll me every 10 milliseconds" for new data.
; In fact, when you put 10 or values near 10 here Windows will poll the device every 8 
; milliseconds.  Further, you can put values as low as 1 millisecond in here and it still works
; in our limited testing.  We don't think you're supposed to use numbers like that though 
; because they chew up USB bandwidth.
; 

POLLING_INTERVAL: EQU 0Ah              ; Polling interval in milliseconds




DecodeRequest:
    mov a, [bmRequestType]              ; is this a standard request ?
    and a, 60h                      ; if not check if its a class request
    jnz ClassRequest

;===================================================================
; Decode Standard Request
;===================================================================

StandardRequest:
    mov a, [bRequest]               
    asl a                           ;multiply index by 2 for jump table
    cmp a, (end_standard_request_table - standard_request_table + 1)
    jnc SendStall                   ; check request is in range

    jacc standard_request_table

XPAGEOFF
standard_request_table:
    jmp GetStatus
    jmp ClearFeature
    jmp SendStall                   ;reserved
    jmp SetFeature              
    jmp SendStall                   ;reserved
    jmp SetAddress
    jmp GetDescriptor
    jmp SetDescriptor
    jmp GetConfiguration
    jmp SetConfiguration
    jmp GetInterface
    jmp SetInterface
end_standard_request_table:             ;synch frame not supported as 63xxx does not support ISO
XPAGEON

;===================================================================
; Decode HID Class Request
;===================================================================

ClassRequest:
    cmp a, 20h                  
    jnz VendorRequest                   ; if not a class request then check for a vendor request

    mov a, [bRequest]                   ; check request is in range
    asl a                           ; multiply index by 2 for jump table
    cmp a, (end_HID_request_table - HID_request_table + 1)
    jnc SendStall

    jacc HID_request_table              ; jump to the routine that handles that request type

    db  00,00,00,00,00,00,00,00,00,00

XPAGEOFF
HID_request_table:              
    jmp SendStall                   ;reserved
    jmp GetReport
    jmp GetIdle
    jmp GetProtocol
    jmp SendStall                   ;reserved
    jmp SendStall                   ;reserved
    jmp SendStall                   ;reserved
    jmp SendStall                   ;reserved
    jmp SendStall                   ;reserved
    jmp SetReport
    jmp SetIdle
    jmp SetProtocol
end_HID_request_table:
XPAGEON

    ; these NOP's allow the flash loader to be downloaded
    ; and for several thousand S-ecords to be sent successfully
    ; presumably because of how they affect paging  - RG
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop


;===================================================================
; Decode Vendor Request
;===================================================================
; Frameworks does not support a vendor request; The user may insert 
; code to do so here

VendorRequest:
    jmp SendStall                   ;no vendor requests supported in Frameworks
    
;=======================================================================================================
;       Handle Request - Get Status
;=======================================================================================================
; Device, Interface and Endpoint are all valid recipients for this request. 
 
GetStatus:
      mov A, 2                              ; send two bytes
      mov [data_count], A

    mov a, [bmRequestType]              ;check recipient, and handle accordingly
    cmp a, DEVICE_TO_HOST
    jz GetDeviceStatus

    cmp a, ENDPOINT_TO_HOST
    jz GetEndpointStatus

    cmp a, INTERFACE_TO_HOST
    jnz SendStall                   ;80, 81, 82 are only valid bmRequestTypes for this request 

;===================================================================
; Recipient = Interface
;===================================================================
; The interface status is a 16-bit value (two bytes) that is always 
; zero for both bytes.

GetInterfaceStatus:
      mov A, (get_interface_status_table - control_read_table)
      jmp SendRomData                       ; send interface status to host

;===================================================================
; Recipient = Device
;===================================================================
; The device status is a 16-bit value (two bytes) with only D[1:0]
; defined.  D0=0 specifies bus-powered, which never changes.  D1
; reflects the status of the device_remote_wakeup feature.  This
; feature can either be disabled (D1=0) or enabled (D1=1).

GetDeviceStatus:
      mov A, (get_dev_status_table - control_read_table)
;     add A, [remote_wakeup_status]         ; get correct remote wakeup **** not yet supported
      jmp SendRomData                       ; send device status to host

;===================================================================
; Recipient = Endpoint
;===================================================================
; The endpoint status is a 16-bit value (two bytes) with only one
; bit (D0) defined.  If D0=0, then the selected endpoint is not
; stalled.  If D0=1, then the selected endpoint is stalled.

GetEndpointStatus:
    iord EP1_TX_Config
    and a, STALL_B
    jz .Send
    mov a, (stalled - get_endpoint_status_table)    ;EP1 is stalled

.Send:
      add A, (get_endpoint_status_table - control_read_table)
      jmp SendRomData                       ; send endpoint status to host

;=======================================================================================================
;       Handle Request - Clear Feature
;=======================================================================================================
; Endpoint and Device are supported recipients for this request. Interface is valid, but not supported.

ClearFeature:
    mov a, [bmRequestType]
    cmp a, HOST_TO_DEVICE
    jz ClearRemoteWakeup            

    cmp a, HOST_TO_ENDPOINT
    jnz SendStall                   ;00, 02 are only valid bmRequestTypes for this request 

;===================================================================
; Recipient = Endpoint
;===================================================================
; Clear the endpoint stall feature for the selected endpoint.  This
; should also set the data 0/1 bit to Data0 if endpoint one is selected.

ClearEndpointStall:
    mov A, [wValue]                         ; 
      cmp A, EP1_STALLED                    ; test for valid feature
      jnz SendStall                         ; stall unsupported features

      call NoDataControl                    ; handshake with host

      iord EP1_TX_Config                    
      and A, ~(STALL_B + DATA10_B)      ; clear stall and data 0/1 bits
      iowr EP1_TX_Config        

      iord USB_SCR                  ; Set up EP1 to NAK INs     
      and A,0EFh                        ; INs are only ACKed when new data is, never clear USBACT BIT
                                    ;Never clear the USBACT bit.
      iowr USB_SCR                  ; ready.

      ret                             

;===================================================================
; Recipient = Device
;===================================================================
; Remote Wakeup is the only Device feature 

ClearRemoteWakeup:
    jmp SendStall                   ;**** not yet supported

    call NoDataControl                  ; handshake with host

      mov A, DISABLE_REMOTE_WAKEUP          ; disable remote wakeup
      mov [remote_wakeup_status], A

      ret                                   

;=======================================================================================================
;       Handle Request - Set Feature
;=======================================================================================================
; Endpoint and Device are supported recipients for this request. Interface is valid, but not supported.

SetFeature:
    mov a, [bmRequestType]
    cmp a, HOST_TO_DEVICE
    jz SetRemoteWakeup          

    cmp a, HOST_TO_ENDPOINT
    jnz SendStall                   ;00, 02 are only valid bmRequestTypes for this request 

;===================================================================
; Recipient = Endpoint
;===================================================================
; Set the endpoint stall feature for the selected endpoint.  

SetEndpointStall:
      mov A, [wValue]                       
      cmp A, EP1_STALLED                    ; test for valid feature
      jnz SendStall                         ; stall unsupported features

      call NoDataControl                    ; handshake with host

      mov A, 30h                            ; stall endpoint one 
      iowr EP1_TX_Config                 
      ret                                   

;===================================================================
; Recipient = Device
;===================================================================
; Remote Wakeup is the only Device feature 
; If the device does not support remote wakeup then replace this code 
; with <jmp SendStall>

SetRemoteWakeup:
      mov A, [wValue]                 
      cmp A, REMOTE_WAKEUP                  ; test for valid feature
      jnz SendStall                         ; stall unsupported features

      call NoDataControl                    ; handshake with host

      mov A, ENABLE_REMOTE_WAKEUP           ; enable remote internal wakeup variable
      mov [remote_wakeup_status], A
      ret                             

;=======================================================================================================
;       Handle Request - Set Address
;=======================================================================================================
; Device is the only valid recipient for this request.

SetAddress:
    mov a, [bmRequestType]
    cmp a, HOST_TO_DEVICE
    jnz SendStall

    mov a, [wValue]                 ;check that the address is in range
    and a, 80h                      ;valid range is 0 to 7F
    jnz SendStall                   ;

      call NoDataControl                    ; handshake with host

.wait:
    iowr wdt
    iord EP0_TX_Config              ; Wait for the ACK.
    and A, 80h                      ; must wait for ACK before changing address
    jnz .wait                       ; as the ACK will be sent to the old address.

      mov A, [wValue]                       ; write new USB device address 
      iowr USB_DA                       ; 
      ret                               

;=======================================================================================================
;       Handle Request - Get Descriptor
;=======================================================================================================
; There are five standard descriptor types and 3 HID class descriptor types. The descriptor type is in 
; the high byte of wValue.  The descriptor index is in the low byte of wValue.  The standard request to 
; a device supports three of these types: device, configuration, and string.  
; The standard request does not support interface or endpoint descriptor types

GetDescriptor:
    mov a, [bmRequestType]
    cmp a, DEVICE_TO_HOST
    jnz GetClassDescriptor              ; If recipient is not device, then check for Interface

      mov A, [wValueHi]                     
    asl a                           ; multiply index by 2 for jump table
    cmp a, (end_desc_type_table - desc_type_table + 1)
    jnc SendStall                   ; check that the descritor type is valid

    jacc desc_type_table

GetClassDescriptor:
    mov a, [bmRequestType]
    cmp a, INTERFACE_TO_HOST
    jnz SendStall                   ; only valid recipient is Interface

      mov A, [wValueHi]                     ; load descriptor type
    and a, dfh                      ; mask class bit    
    asl a                           ; multiply index by 2 for jump table
    cmp a, (end_class_desc_type_table - class_desc_type_table + 1)
    jnc SendStall

    jacc class_desc_type_table



XPAGEOFF
desc_type_table: 
    jmp SendStall                   ;no descriptor index 0
    jmp GetDeviceDescriptor 
    jmp GetConfigurationDescriptor  
    jmp GetStringDescriptor 
end_desc_type_table:

class_desc_type_table:  
    jmp SendStall       ; no descriptor index 0
    jmp GetHIDDescriptor    
    jmp GetReportDescriptor 
    jmp GetPhysicalDescriptor   
end_class_desc_type_table:
XPAGEON

;===================================================================
; Get Descriptor = Device
;===================================================================

GetDeviceDescriptor:
      mov A, (end_device_desc_table - device_desc_table)           
      mov [data_count], A                   ; save the device descriptor length

      mov A, (device_desc_table - control_read_table)
      jmp SendRomData                       ; pass a pointer to the device descriptor data

;===================================================================
; Get Descriptor = Config
;===================================================================
; When requested to send the config descritor all of the Class, Interface, 
; HID and Endpoint descritors should be sent. For Draft 4 compliance, this
; order in mandatory

GetConfigurationDescriptor:
    mov A, (end_config_desc_table - config_desc_table)
      mov [data_count], A                       ; save the config descriptor length

    mov A, (config_desc_table - control_read_table)
    jmp SendRomData             ; pass a pointer to the config descriptor data

;===================================================================
; Get Descriptor = String
;===================================================================

GetStringDescriptor:               
    mov A, [wValue]                 ; get string descriptor index
    cmp a, (end_string_length_table - string_length_table + 1)
    jnc SendStall                   ; check index is in range

    index string_length_table           ; get the descriptor length
      mov [data_count], A                       ; 

    mov A, [wValue]                 ; get string descriptor index
    index string_offset_table           ; get a pointer to the string descriptor data
      jmp SendRomData                       ; 
    

XPAGEOFF
string_length_table:
    db (USBStringDescription1 - USBStringLanguageDescription)
    db (USBStringDescription2 - USBStringDescription1)
    db (USBStringDescription3 - USBStringDescription2)
    db (USBStringDescription4 - USBStringDescription3)
    db (USBStringDescription5 - USBStringDescription4)
    db (USBStringEnd - USBStringDescription5)
end_string_length_table:

string_offset_table:
    db (USBStringLanguageDescription - control_read_table)
    db (USBStringDescription1 - control_read_table)
    db (USBStringDescription2 - control_read_table)
    db (USBStringDescription3 - control_read_table)
    db (USBStringDescription4 - control_read_table)
    db (USBStringDescription5 - control_read_table)
end_string_offset_table:
XPAGEON

;===================================================================
; Get Descriptor = Report
;===================================================================

GetReportDescriptor:
    mov A, (end_hid_report_desc_table - hid_report_desc_table)
      mov [data_count], A                   ; save descriptor length            

    mov A, (hid_report_desc_table - control_read_table)
      jmp SendRomData                       ; send descriptor to host

;===================================================================
; Get Descriptor = HID
;===================================================================

GetHIDDescriptor:
    mov A, (Endpoint_Descriptor - Class_Descriptor)
      mov [data_count], A                   ; save descriptor length            

    mov A, ( Class_Descriptor - control_read_table)
      jmp SendRomData                       ; send descriptor to host

;===================================================================
; Get Descriptor = Physical
;===================================================================
; Physical descriptors are not supported by frameworks. Users wishing
; to support them should add code to send them here.

GetPhysicalDescriptor:
    jmp SendStall                           ;not supported by frameworks

;=======================================================================================================
;       Handle Request - Set Descriptor
;=======================================================================================================
; Set Descritor is not supported by frameworks. Users wishing to support this request should add code to 
; do so here.

SetDescriptor:                              ;not supported by frameworks
    jmp SendStall

;=======================================================================================================
;       Handle Request - Get Configuration
;=======================================================================================================
; The only recipient supported for this request is Device
;
; Returns the current device configuration. Valid values are zero (unconfigured) and one (configured).

GetConfiguration:
    mov a, [bmRequestType]
    cmp a, DEVICE_TO_HOST
    jnz SendStall

      mov A, 1                              ; send one byte
      mov [data_count], A

      mov A, (get_configuration_status_table - control_read_table)
      add A, [configuration_status]         ; get correct configuration

      jmp SendRomData                       ; send configuration to host

;=======================================================================================================
;       Handle Request - Get Configuration
;=======================================================================================================
; The only recipient supported for this request is Device
;
; Sets the configuration of the device to either unconfigured (0) or configured (1) based on wValue in the 
; SETUP packet. USB spec rev 1.1 (para 9.1.1.5), requires Set Configuration to also clear the endpoint stall 
; condition and re-initialize endpoints using data 0/1 toggle to Data0.

SetConfiguration:
    mov a, [bmRequestType]
    and a, HOST_TO_DEVICE                   ; faster than <cmp a, 00h>
    jnz SendStall                           ; stall if recipient not Device

      call NoDataControl                    ;handshake with host

      mov A, [wValue]           
      mov [configuration_status], A         ; store configuration byte

      iord EP1_TX_Config                    ; clear data 0/1 bit
      and A, ~ (DATA10_B + STALL_B)
      iowr EP1_TX_Config    

      mov A, [configuration_status]
      and a, ffh                            ; faster than <cmp a, 00h>
      jnz ConfigureDevice

UnconfigureDevice:
    iord EP1_TX_Config
    and A, ~INEN_B                          ; disable endpoint 1
    iowr EP1_TX_Config            

    mov A, [interrupt_mask]                 ; disable endpoint one interrupts
    and A, ~EP1IE_B
    mov [interrupt_mask], A

    ret

ConfigureDevice:
    iord EP1_TX_Config                      ; NAK IN packets until data is  
    and A,7Fh                               ; ready on endpoint one
    or A, 10h                               ; enable endpoint one
    iowr EP1_TX_Config            

    mov A, [interrupt_mask]                 ; enable gpio, ep1, 1ms 128us interrupts
    or A, ENUMERATED_MASK
    mov [interrupt_mask], A

    iord USB_SCR                            ; NAK IN packets until data is  
    and A,0EFh                              ; ready on endpoint one, Never clear USBACT bit
    iowr USB_SCR              

    ret                             

;=======================================================================================================
;       Handle Request - Get Interface
;=======================================================================================================
; The only recipient supported for this request is Interface
;
; The interface is an 8 bit value that is always zero since there are no alternate settings defined

GetInterface:
    mov a, [bmRequestType]
    cmp a, INTERFACE_TO_HOST
    jnz SendStall

    mov A, [wIndex]                         ;which interface ?
      cmp A, 0                              ;only interface 0 defined
      jnz SendStall

      mov A, 1                              ;send one byte
      mov [data_count], A

      mov A, (get_interface_table - control_read_table)
      jmp SendRomData                        ;send alternate interface to host

;=======================================================================================================
;       Handle Request - Set Interface
;=======================================================================================================
; The only recipient supported for this request is Interface
;
; The host may attempt to set the interface to an alternative setting. The frameworks does not support any 
; alternate setting, but user firmware may so we do not stall that request, though it would be legal to do so.

SetInterface:
    mov a, [bmRequestType]
    cmp a, DEVICE_TO_HOST
    jnz SendStall

    mov a,[wIndex]                  ; which interface ?
    cmp a,0                     ; only interface 0 defined
    jnz SendStall

    mov a,[wValue]                  ; which setting
    cmp a, 0                        ; only setting 0 supported
    jnz SendStall

    jmp NoDataControl                   ; ack the request

;=======================================================================================================
;       Handle Request - Get Report
;=======================================================================================================
; This is a HID Class request. The only recipient supported is Interface
;
; Get Report allows the host to receive a report via the control pipe. The report type is specified in 
; the wValue high byte while the low byte has a report ID.  

GetReport:
    mov a, [bmRequestType]
    cmp a, CLASS + INTERFACE_TO_HOST
    jnz SendStall  ;Send a stall if this is not valid request

; get report support not needed

    jmp SendStall


;=======================================================================================================
;       Handle Request - Get Idle
;=======================================================================================================
; This is a HID Class request. The only recipient supported is Interface
;
; Microsoft inform us that this request is not used. As it is optional, it is stalled.

GetIdle:
      jmp SendStall         

;=======================================================================================================
;       Handle Request - Get Protocol
;=======================================================================================================
; This is a HID Class request. The only recipient supported is Interface
;
; Get Protocol sends the current protocol status back to the host. For boot protocol, wValue=0.  
; For report protocol, wValue=1.

GetProtocol:
    mov a, [bmRequestType]
    cmp a, CLASS + INTERFACE_TO_HOST
    jnz SendStall

      mov A, 1                              ; send one byte
      mov [data_count], A

      mov A, (get_protocol_status_table - control_read_table)
      add A, [protocol_status]              ; get correct configuration
      jmp SendRomData                       
      
;=======================================================================================================
;       Handle Request - Set Report
;=======================================================================================================
; This is a HID Class request. The only recipient supported is Interface
;
; Set Report allows the host to transmit a report via the control pipe. The report type is specified in 
; the wValue high byte while the low byte has a report ID.  

SetReport:
    mov a, [bmRequestType]
    cmp a, CLASS + HOST_TO_INTERFACE
    jnz SendStall

    ;DETERMINE THE REPORT ID OF THIS SETREPORT

    mov a, [wValue]                         ;Read the current REPORTID and save it for later.
    mov [current_reportid], a 
       
    mov a, [wLength]                        ;Read the descriptor Length and save for later.  
    mov [data_count], a                     ;Use it to count # bytes needed to read.

    ;CLEAR THE SETUP BIT
    mov a, 00h                              ;The SETUP bit must be cleared so that only the OUT
    iowr EP0_RX_Status                      ;bit is set with the out packet comes around on the
                                            ;next interrupt.

    mov a, [current_reportid]
    cmp a, RID_TRANSMIT                         ; is it a transmit serial data command?
    jz hid_report_incoming                      ;  yes, set the flags and such to do this

    cmp a, RID_COMMAND                          ; is it a command to change baud rate?
    jnz await_outs                              ;   no, its something else
                                                ;   yes, set the flags and such to do this

hid_report_incoming:
    mov a, TRANSMITTING_MASK
    mov [interrupt_mask],a

    mov a, 0                            ; disable startbit interrupts
	iowr P0_interrupt           

IFDEF SOFTWARE_FLOW_CONTROL
    ; flow off the target.  if we don't it may try to transmit data while we're transmitting,
    ; and that's not good:  we're a half duplex device.

    mov a,XOFF                              ;Load the serial register with an XOFF
    mov [serial_tx_reg],a                   ;and transmit it to the device
    call SerialTransmitByte                 ;Transmit the XOFF
ENDIF

    FLOW_OFF                                ; turn of flow (hardware)

    mov a,FIRSTBYTE_FLG_MASK                ;Indicate that we will receive the first byte in the data
    or [fwcr], a                             ;sequence.  This flag is used to avoid sending the RID

await_outs:
                                       
    iord USB_SCR                            ; enable OUT pkts to go into the FIFO again
    and a, ~STATOUTS_B
    or A, ENOUTS_B
    iowr USB_SCR

    ret


;=======================================================================================================
;       Handle Request - Set Idle
;=======================================================================================================
; This is a HID Class request. The only recipient supported is Interface
;
; Set Idle silences a particular report on the interrupt pipe until a new event occurs or the specified 
; amount of time (wValue) passes.
;
; Microsoft inform us that this request is not used. As it is optional, it is stalled.

SetIdle:
    jmp SendStall   

;=======================================================================================================
;       Handle Request - Get Protocol
;=======================================================================================================
; This is a HID Class request. The only recipient supported is Interface
;
; Set Protocol switches between the boot protocol and the report protocol. For boot protocol, wValue=0.  
; For report protocol, wValue=1.

SetProtocol:
    mov a, [bmRequestType]
    cmp a, CLASS + HOST_TO_INTERFACE
    jnz SendStall

      mov A, [wValue]                 
      mov [protocol_status], A              ; write new protocol value

      jmp NoDataControl                     ; handshake with host

;=======================================================================================================
;********** USB library main routines *******************
;=======================================================================================================



;===================================================================
; Function - Send ROM Data
;===================================================================
; This function controls the sending of data from the descriptor table
; starting from the table index passed in the accumulator. The number
; of bytes to be sent is passed using the global variable <data_count>
; This can be something larger than 8 bytes. (For instance, a descriptor table)
;

SendRomData:                                    
      mov [data_start], A                   ; save start index

      call GetDescriptorLength              ; correct the descriptor length
      call ControlRead                      ; send the data

      ret                             

;===================================================================
; Function Send Stall
;===================================================================
; Send a stall to indicate the requested function is not supported

SendStall:              
      mov A, INEN_B + STALL_B               ;   
      iowr EP0_TX_Config                    ; set the EP0 stall bit
        
      ret                   

;===================================================================
; Function Get Descriptor Length
;===================================================================
; The host sometimes lies about the number of bytes it wants from a 
; descriptor.  Any request to get descriptor should return the lesser 
; of the number of bytes requested or the actual length of the descriptor.
;
; INPUTS:  
;   data_count  : Number of bytes
;   wLength     : EP0_FIFO + 6 
;   wLengthHi   : EP0_FIFO + 7 (High byte of FIFO)
;

GetDescriptorLength:
    mov A, [wLengthHi]               
    cmp A, 0                                ; confirm high byte is zero
    jnz UseActualLength                     ; no requests should be longer than 256b
        
    mov A, [wLength]                        ; test low byte against zero
    cmp A, 0
    jz UseActualLength                      ; must request some data

    cmp A, [data_count]                     ; compare to the amount of data
    jnc UseActualLength

    mov [data_count], A                     ; use requested length

UseActualLength:
      ret                   




;===================================================================
; Function - NoDataControl
;===================================================================
; Send a 0 length data packet to ACK the data sent by the host. 

NoDataControl:
    iord EP0_RX_Status
    and a, ~SETUP_B
    iowr EP0_RX_Status
                                 
    mov A, INEN_B + DATA10_B                ; set up the transfer register for data1
    iowr EP0_TX_Config                      ; and 0 byte transfer

    ret             

;===================================================================
; Function - Control Read
;===================================================================
;   Purpose:   Performs the control read operation
;          as defined by the USB specification
;   SETUP-IN-IN-IN...OUT

;   data_count: must be set to the size of the 
;           descriptor 

ControlRead: 
    mov A, 00h                              ; clear data 0/1 bit, Always start with a DATA0 label
    mov [endp0_data_toggle], A              

ControlReadDataStage:
                                            ; WIll only deal with 8 bytes. (8 bytes/packet)
                                            ; therefore for N packets, must enter this code
                                            ; N times. (This includes the setup data)
    push X              
    mov X, 00h
    mov A, 00h

    mov [loop_counter], A                   ; set loop counter to 00
    iowr EP0_RX_Status                      ; clear setup bit
        
    iord EP0_RX_Status                      ; check to make sure another setup
    and A, 01h                              ; has not arrived. Id so, exit ISR
    jnz ControlReadStatusStage          

                                            ;set BADOUTS bit and Don't clear USBACT bit.
    iord USB_SCR
    or a, STATOUTS_B                        ;Set controller to respond automatically to status out stage.
    iowr USB_SCR

    mov A, [data_count]
    cmp A, 00h
    jz DmaLoadDone

DmaLoadLoop:                                ; loop to load data into the data buffer
    mov A, [data_start]
    index control_read_table
    mov [X + EP0_FIFO], A                   ; load dma buffer

    inc [data_start]
    inc X
    inc [loop_counter]

    dec [data_count]                    ; Exit if descriptor is done
    jz DmaLoadDone                      ; even if packet has less than 8 bytes data.

    mov A, [loop_counter]               ; or 8 bytes sent
    cmp A, 08h
    jnz DmaLoadLoop

DmaLoadDone:
    iord EP0_RX_Status                  ; check setup bit
    and A, 01h                          ; if not cleared, another setup
    jnz ControlReadStatusStage          ; has arrived so exit

    mov A, [endp0_data_toggle]
    xor A, DATA10_B
    mov [endp0_data_toggle], A

    or A, 80h                           ; Setting bit 7 so that device can respond to an IN
    or A, [loop_counter]
    iowr EP0_TX_Config

ControlReadStatusStage:                 ; OUT at end of data transfer
    pop X                               ; restore X from stack
    ret   
                              
;===================================================================
; Function - Control Write
;===================================================================
;   Purpose:   Performs the control write operation
;          as defined by the USB specification
;   SETUP-OUT-OUT-OUT..IN
;
;  This is a result of a non-NULL OUT token received with associated data. 
;


ControlWrite: 

ControlWriteRamDataStage:
    push X              

    mov a, [current_reportid]
    cmp a, RID_COMMAND                  ; is it a command to change baud rate?
    jnz ridtransmit
    
    mov a, [EP0_FIFO+1]                 ; put command parameter into A
    call handle_rid_command
    jmp cwhandshake

ridtransmit:
    iord EP0_RX_Status                  ; get count of bytes in this OUT packet (in upper four bits)
    rrc                                 
    rrc
    rrc
    rrc
    and a, 0fh                          ; mask off any junk now in upper bits
    sub a, 02h                          ; remove 2 checksum bytes always there
    mov [loop_counter], A               ;Load the loop counter with 00. (Will increment to 8)
    
    mov X, 00h                          ;Set the index for reading the USB FIFO

    ; the first byte in the first OUT packet (following the SETUP) is just the report ID
    ; repeated.  Ignore it.
    
    mov a, [fwcr]                           ;If this is the first data packet of a sequence,
    and a,FIRSTBYTE_FLG_MASK                ;skip over the first byte. This byte contains the
    jz DmaWriteRamLoop                      ;RID of the corresponding SetReport

    mov a, [fwcr]
    and a, ~FIRSTBYTE_FLG_MASK              ; we've dealt with the first byte
    or a, BUSY_TRANSMITTING                 ; inform background that we'll be hogging cycles
    mov [fwcr], a

    mov a, [X+EP0_FIFO]                     ; is this a serial tx report?
    cmp a, RID_TRANSMIT
    jnz cwhandshake                         ;    no, just handshake and leave

    dec [data_count]                        ; decrement our overall count of bytes in this HID report
    dec [loop_counter]                      ; decrement our count of bytes in this packet
    inc X                                   ; 2nd byte in first OUT holds tx byte count

    mov a, [X+EP0_FIFO]
    mov [transmit_count], a

    dec [data_count]                        ; decrement our overall count of bytes in this HID report
    dec [loop_counter]                      ; decrement our count of bytes in this packet
    inc X                                   ; point to next possible byte to tx

	mov a, INTDISABLE_MASK			        ; we don't want interrupts messing w/
	iowr gier                               ; serial tx timing

DmaWriteRamLoop:                        
    mov a, [transmit_count]                 ; have we received all the data we want to tx?
    cmp a, 0                                ; just do this to update Z flag
    jz DmaWriteRam_next

    iowr wdt                                ; kick the dog for each byte transmitted
    mov A, [X + EP0_FIFO]                   ;Send 1-byte of serial data. This function will 

    mov [serial_tx_reg], A                  ;execute for one byte time
    call SerialTransmitByte
    dec [transmit_count]                    ; decrement our count of the bytes we actually want to tx

DmaWriteRam_next:
    inc X
    dec [data_count]                        ; decrement our overall count of bytes in this HID report
    dec [loop_counter]                      ; decrement our count of bytes in this packet
    jnz DmaWriteRamLoop                     ; as long as we have bytes to tx in this packet go try

DmaWriteRamDone:  
    mov A, [data_count]                     ; if 0 then we've just gotten last bytes of this report
    cmp A,00h                               ;   if yes we handshake
    jnz cwenableouts                        ;   if no, exit and be ready for next OUT packet.

cwhandshake:
    call NoDataControl                      ;Handshake the status stage.  Sets up EP0 transmitter to send a null packet
                                            ;upon receiving an IN token.
cwenableouts:
    iord USB_SCR                            ; enable OUT pkts to go into the FIFO again
    or A, ENOUTS_B
    iowr USB_SCR

cwexit:
    pop X                                   
    ret


;=======================================================================================================
; ROM Tables
;=======================================================================================================
;

XPAGEOFF

control_read_table:
device_desc_table:
    db (end_device_desc_table - device_desc_table)         ; Descriptor length (18 bytes)
    db 01h                              ; Descriptor type (Device)
    db 10h,01h                          ; Complies with USB Spec. Release (1.1)
    db 00h                              ; Class code (0)
    db 00h                              ; Subclass code (0)
    db 00h                              ; Protocol (No specific protocol)
    db 08h                              ; Max. packet size for EP0 (8 bytes)
    DWL MOTO_VID                        ; vendor ID
    DWL MOTO_PID                        ; product ID
    DWL MOTO_VER                        ; firmware version (BCD)
    db 01h                              ; Mfr string descriptor index 
    db 02h                              ; Product string descriptor index 
    db 00h                              ; Serial Number string descriptor index (None)
    db 01h                              ; Number of possible configurations (1)
end_device_desc_table:


config_desc_table:
    db (Interface_Descriptor - config_desc_table)         ; Descriptor length (9 bytes)
    db 02h                              ; Descriptor type (Configuration)
    db (end_config_desc_table - config_desc_table) ,00h      
                                        ; Total length of config, i/f, HID, and EP descriptors
    db 01h                              ; Interface supported (1)
    db 01h                              ; Configuration value (1)
    db 04h                              ; Index of string descriptor 
    db 80h                              ; Configuration (Bus powered)
    db 32h                              ; Max power consumption MUST be < 100mA if bus powered

Interface_Descriptor:
    db (Class_Descriptor - Interface_Descriptor)          ; Descriptor length (9 bytes)
    db 04h                              ; Descriptor type (Interface)
    db 00h                              ; Number of interface (0) 
    db 00h                              ; Alternate setting (0)
    db 01h                              ; Number of interface endpoints (1)
    db 03h                              ; Class code ()                    
    db 00h                              ; Subclass code ()                 
    db 00h                              ; Protocol code ()
    db 05h                              ; Index of string()       

Class_Descriptor:
    db (Endpoint_Descriptor - Class_Descriptor) ; Descriptor length (9 bytes)
    db 21h                              ; Descriptor type (HID)
    db 00h,01h                          ; HID class release number (1.00)
    db 00h                              ; Localized country code (None)
    db 01h                              ; # of HID class dscrptr to follow (1)
    db 22h                              ; Report descriptor type (HID)
    dwl (end_hid_report_desc_table - hid_report_desc_table) ; Total length of report descriptor (2 bytes)

Endpoint_Descriptor:
    db (end_config_desc_table - Endpoint_Descriptor)    ; Descriptor length (7 bytes)   
    db 05h                              ; Descriptor type (Endpoint)
    db 81h                              ; Encoded address (Respond to IN, 1 endpoint)
    db 03h                              ; Endpoint attribute (Interrupt transfer)
    db EP1_REPORT_LENGTH+1,00h          ; Maximum packet size (report len + id = 8 bytes)
    db POLLING_INTERVAL                  ; Polling interval



end_config_desc_table:

; This report descriptor is taken from Appendix A12 of the HID Uasage Tables


hid_report_desc_table:   
    db 06h,  00h, ffh          ; USAGE_PAGE (Vendor Defined Page 1)

    db 09h,  01h               ; USAGE (Vendor Usage 1)
    db a1h,  01h               ; COLLECTION (Application)

    db 85h,  RID_RECEIVE       ; REPORT_ID (1)  - streaming UART rx data
    db  9h,  1h                ; USAGE (Vendor Usage 1)
    db 15h,  0h                ; LOGICAL_MINIMUM (0)
    db 26h,  ffh, 0h           ; LOGICAL_MAXIMUM (255)
    db 75h,  8h                ; REPORT_SIZE (8) = bits 
    db 95h,  EP1_REPORT_LENGTH ; REPORT_COUNT (7) = 1 report id byte + 7 data bytes
    db 81h,  6h                ; INPUT (Data,Var,Rel)
    db  9h,  1h                ; USAGE (Vendor Usage 1)   
    db b1h,  6h                ; FEATURE (Data,Var,Rel) 

    db 85h,  RID_COMMAND       ; REPORT_ID (3) Command (setreport)
    db 09h,  01h               ; USAGE (Vendor Usage 1)
    db 15h,  00h               ; LOGICAL_MINIMUM (0)
    db 26h,  8h, 00h           ; LOGICAL_MAXIMUM (255)
    db 75h,  08h               ; REPORT_SIZE (8) bits
    db 95h,  01h               ; REPORT_COUNT (1) byte
    db 09h,  01h               ; USAGE (Vendor Usage 1)
    db b1h,  06h               ; FEATURE (Data,Var,Rel) 
        
    db 85h,  RID_TRANSMIT      ; REPORT_ID (2) serial transmit these bytes (SetReport)
    db 09h,  03h               ; USAGE (Vendor Usage 1)
    db 96h,  07h, 00h          ; REPORT_COUNT (7) 1 count byte, 6 data
    db b1h,  06h               ; FEATURE (Data,Var,Rel)   

    db c0h                     ; END_COLLECTION
end_hid_report_desc_table:

;========================================================================
; These tables are the mechanism used to return status information to the
; host.  The status can be either device, interface, or endpoint.

get_dev_status_table:
        db 00h, 00h        ; remote wakeup disabled, bus powered
wakeup_enabled:      
        db 02h, 00h        ; remote wakeup enabled, bus powered

get_interface_status_table:
        db 00h, 00h        ; always return both bytes zero

get_endpoint_status_table:
        db 00h, 00h        ; not stalled
stalled:
        db 01h, 00h        ; stalled

get_configuration_status_table:
        db 00h             ; not configured
        db 01h             ; configured

get_protocol_status_table:
        db 00h             ; boot protocol
        db 01h             ; report protocol

get_interface_table:
        db 00h             ; no alternate setting

;========================================================================
; String Descriptors        Do not make the string descriptor too long
;                           because the entire rom lookup table cannot be
;                           longer than 256 bytes

; string 0
USBStringLanguageDescription:
    db (USBStringDescription1 - USBStringLanguageDescription)         
    db 03h          ; Type (3=string)
    db 09h          ; Language:  English
    db 04h          ; Sub-language: Default

; string 1
USBStringDescription1:  ; IManufacturerName
    db (USBStringDescription2 - USBStringDescription1)         
    db 03h          ; Type (3=string)
    dsu "Moto" ;

; string 2
USBStringDescription2:  ; IProduct
    db (USBStringDescription3 - USBStringDescription2)         
    db 03h          ; Type (3=string)
    dsu "MO1002"  ;

;string 3
USBStringDescription3:  ; serial number
                        ; If a SN is used, this must be unique
                        ; for every device or the device may
                        ; not enumerate properly
; string 4                 
USBStringDescription4:  ; configuration string descriptor
    db (USBStringDescription5 - USBStringDescription4)         
    db 03h          ; Type (3=string)
    dsu "2400"   ;

;string 5
USBStringDescription5:  ; interface string descriptor
    db (USBStringEnd - USBStringDescription5)         
    db 03h          ; Type (3=string)
    dsu "USB ART"  ;

USBStringEnd:
XPAGEON

